/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  Nuvoton Technoledge Corp creadted by PU20 
  This demo code is used to show the external interrupt function.    
                                            
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0xF
#define PLL_Engine_Enable     1 
#define PLL_SEL               0x00000000 
#define CLOCK_SEL             0x0

#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "retarget.h"

/*-----------------------------------------------------------------------------
  Function:		EINT0_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	EINT0 interrupt ISR.                                    
 *-----------------------------------------------------------------------------*/
void EINT0_IRQHandler(void)
{
	P3_ISRC = P3_ISRC;
	P0_DOUT = ~P0_DOUT;
}

main(void)
{
    Un_Lock_Reg();
	PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock.

	P3_MFP = (P3_MFP & (~P32_EINT0)) | EINT0;	//Set P3.2 as external interrupt function.
	
	DBNCECON &= ~ICLK_ON;
	DBNCECON &= DBCLK_HCLK;
	DBNCECON |= SMP_256CK;						//Set De-bounce clock cycle.
												  
	P3_DBEN |= DBEN2;							//Enable P3.2 De-bounce function.
		
	P3_IMD &= IMD2_EDG;						
	P3_IEN |= IF_EN2;							//Set EINT0 falling edge trigger.
	NVIC_ISER |= EXT_INT0;

	P0_DOUT = 0x00;

    while(1);
}
