/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Uart.h"
#include "L_Function.h"

/*-----------------------------------------------------------------------------
  Function:		TMR0_Delay1ms                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulCNT:	delay time, unit is millisecond.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 used for a delay function.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Delay1ms(uint32_t ulCNT)
{
    TCMPR0 = 12000;         //Fosc=12MHz, so 12000000/12000=1000Hz=1ms
    setTMR0_CEN;            //start timer0
    while (ulCNT != 0)
    {
        while ((TISR0&TMR_TIF) != TMR_TIF); //check TIF0
        TISR0 |= TMR_TIF;   //Clear TIF0
        ulCNT --;
    }
    clrTMR0_CEN;            //stop timer0
}

void Long_Delay(void)
{
    uint32_t ulCNT;
    uint8_t uiTmp;
    for(ulCNT=0;ulCNT<0x1FFFF;ulCNT++)
        uiTmp ++;
}

/*-----------------------------------------------------------------------------
  Function:		Timer0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0, period mode.                                    
 *-----------------------------------------------------------------------------*/
void Timer0_Init(void)
{
    TMR0_Clock_EN;
	TMR0ClkSource_ex12MHz;
    TCSR0  = 0x00000000;    //Pre-Scaler
    
    setTMR0_PERIOD;
    setTMR0_IE;             //Timer0 interrupt enable
    setTMR0_CRST;           //reset the timer/counter0, after set, this bit will be clear by H/W
}
