/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Retarget.h"
#include "Uart.h"
#include "Common.h"
#include "Macro_SPI.h"
#include "L_SPI.h"

uint32_t  SPI0_RX0_Data = 0, SPI1_RX0_Data = 0;

extern uint32_t ucgSPI0_Divider_Value,ucgSPI1_Divider_Value;

/*-----------------------------------------------------------------------------
  Function:		SPI0_Master_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize SPI0 as master.                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Master_Init(void)
{                                     
    P1_MFP &= ~(P14_AIN4_SPI0SS |
				P15_AIN5_SPI0MOSI |
				P16_AIN6_SPI0MISO |
				P17_AIN7_SPI0CLK) ;  
	
	P1_MFP |= (SPI0SS |
			   SPI0MOSI |
			   SPI0MISO |
			   SPI0CLK) ;   			//Enable SPI0 Pin in I/O

	ENABLE_SPI0_CLK; 

    						      		//             ___          ____ 
    SPI0_SSR &= ~LVL_H;        			//SS_LVL = 0     l_________l

    SPI0_CNTRL &= ~LSB_FIRST; 			//LSB  = 0, MSB Transmit/Receiv first.
    SPI0_CNTRL &= ~CLKP_IDLE_H;    		//CLKP = 0, SCLK Idel Low
    SPI0_CNTRL |= TX_NEG_F;     		//Tx_NEG = 1, The SDO signal is changed on the falling edge.
    SPI0_CNTRL &= ~RX_NEG_F;     		//Rx_NEG = 0, The SDI signal is latched on the rising edge.

    CLKDIV &= 0xFFFFFFF0;               //HCLK_N = 0, Pclk = SYSclk/(HCLK_N+1)
    SPI0_DIVIDER &= 0xFFFF0000;         //SPIclk = Pclk/((HCLK_N+1)*2)
    SPI0_DIVIDER |= ucgSPI0_Divider_Value;

    SET_SPI0_MASTER_MODE;           	//SPI = Master Mode
    ENABLE_SPI0_AUTO_SLAVE_SLECT;   	//Enable Automatic Slave Select
    SPI0_SSR |= SSR_ACT;			    //Auto Chip Slave Select Register on SPI0.0, SPI0.1
}

/*-----------------------------------------------------------------------------
  Function:		SPI0_Send_1W                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulData: data to be sent
				ucLength: SPI0 transmit bit length, range is 0x00~0x20                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	SPI0 send a data                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Send_1W(uint32_t ulData, uint8_t ucLength)
{
    SPI0_CNTRL &= TX_NUM_ONE;     		//Only one Tx0/Rx0
    SPI0_Length(ucLength);
    SPI0_TX0 = ulData;
    SPI0_CNTRL |= GO_BUSY;     	  		//Start Transfer
}

/*-----------------------------------------------------------------------------
  Function:		Get_Slave_ID                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	SPI0 will receive 0xDA.                                   
 *-----------------------------------------------------------------------------*/
void Get_Slave_ID(void)
{
    //Send Get ID Command
    SPI0_Send_1W(0x00000055, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);

    //Send NULL Data for Get Feed Back
    SPI0_Send_1W(0x00000000, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);
    SPI0_RX0_Data = SPI0_RX0;
}

/*-----------------------------------------------------------------------------
  Function:		SPI0_Length                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ucLength: SPI0 transmit bit length, range is 0x00~0x20                                      
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Set SPI0 transmit data length.                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Length(uint8_t ucLength) 
{
     if(ucLength <= 0x20)
    {
        if((ucLength & 0x01) == 0)
            SPI0_CNTRL &= ~(1<<3);
        else
            SPI0_CNTRL |= (1<<3);
        if((ucLength & 0x02) == 0)
            SPI0_CNTRL &= ~(1<<4);
        else
            SPI0_CNTRL |= (1<<4);

        if((ucLength & 0x04) == 0)
            SPI0_CNTRL &= ~(1<<5);
        else
            SPI0_CNTRL |= (1<<5);

        if((ucLength & 0x08) == 0)
            SPI0_CNTRL &= ~(1<<6);
        else
            SPI0_CNTRL |= (1<<6);

        if((ucLength & 0x10) == 0)
            SPI0_CNTRL &= ~(1<<7);
        else
            SPI0_CNTRL |= (1<<7);
    } // end if(ucLength < 0x20)
}

/*-----------------------------------------------------------------------------
  Function:		SPI1_Slave_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize SPI1 as slave.                                    
 *-----------------------------------------------------------------------------*/
void SPI1_Slave_Init(void)
{                               
    P0_MFP &= ~(P04_AD4_SPI1SS |
				P05_AD5_SPI1MOSI |
				P06_AD6_SPI1MISO |
				P07_AD7_SPI1CLK) ;  
	
	P0_MFP |= (SPI1SS |
			   SPI1MOSI |
			   SPI1MISO |
			   SPI1CLK) ;   			//Enable SPI1 Pin in I/O

	ENABLE_SPI1_CLK;	

	SPI1_SSR &= LTRIG_EDG;       		//Edge-Trigger
    SPI1_SSR &= ~LVL_H;       			//Slave Select Signal Active Low.

    SPI1_CNTRL &= ~LSB_FIRST;    		//LSB  = 0, MSB Transmit/Receiv first.
    SPI1_CNTRL &= ~CLKP_IDLE_H;    		//CLKP = 0, SCLK Idel Low
    SPI1_CNTRL |= TX_NEG_F;     		//Tx_NEG = 1, The SDO signal is changed on the falling edge.
    SPI1_CNTRL &= ~RX_NEG_F;     		//Rx_NEG = 0, The SDI signal is latched on the rising edge.

    CLKDIV &= 0xFFFFFFF0;               //HCLK_N = 0, Pclk = SYSclk/(HCLK_N+1)
    SPI1_DIVIDER &= 0xFFFF0000;         //SPIclk = Pclk/((HCLK_N+1)*2)
    SPI1_DIVIDER |= ucgSPI1_Divider_Value;
    SET_SPI1_SLAVE_MODE;        		//SPI = Slave Mode
    
    ENABLE_SPI1_INTERRUPT;      		//Eable Interrupt
    NVIC_ISER |= SPI1_INT;     			//NVIC Enable SPI1 Interrupt 
    SPI1_Length(8);
    SPI1_GO_BUSY;               		//Waiting for SPI1 RX0 Signal
}

/*-----------------------------------------------------------------------------
  Function:		SPI1_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	SPI1 ISR routine.                                    
 *-----------------------------------------------------------------------------*/
void SPI1_IRQHandler(void)
{
    SPI1_CNTRL |= SPI_IF;    			//Clear Interrupt Flag
    if(SPI1_RX0 == 0x55)
    {
        SPI1_RX0_Data = SPI1_RX0;
        SPI1_TX0 = 0x000000DA;
    }
    else
        SPI1_TX0 = 0x00000000;
    SPI1_GO_BUSY;               		//Waiting for SPI1 RX0 Signal
}

/*-----------------------------------------------------------------------------
  Function:		SPI1_Length                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ucLength: SPI1 transmit bit length, range is 0x00~0x20                                      
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Set SPI1 transmit data length.                                    
 *-----------------------------------------------------------------------------*/
void SPI1_Length(uint8_t ucLength) 
{
     if(ucLength <= 0x20)
    {
        if((ucLength & 0x01) == 0)
            SPI1_CNTRL &= ~(1<<3);
        else
            SPI1_CNTRL |= (1<<3);
        if((ucLength & 0x02) == 0)
            SPI1_CNTRL &= ~(1<<4);
        else
            SPI1_CNTRL |= (1<<4);

        if((ucLength & 0x04) == 0)
            SPI1_CNTRL &= ~(1<<5);
        else
            SPI1_CNTRL |= (1<<5);

        if((ucLength & 0x08) == 0)
            SPI1_CNTRL &= ~(1<<6);
        else
            SPI1_CNTRL |= (1<<6);

        if((ucLength & 0x10) == 0)
            SPI1_CNTRL &= ~(1<<7);
        else
            SPI1_CNTRL |= (1<<7);
    } // end if(ucLength < 0x20)
}

/*-----------------------------------------------------------------------------
  Function:		Print_Get_Data                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                      
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print SPI1 received command and data return to SPI0.                                    
 *-----------------------------------------------------------------------------*/
void Print_Get_Data(void)
{
    printf("Slave Get Command = %X\n", SPI1_RX0_Data);
    printf("Master Get Feed Back Data = %X\n", SPI0_RX0_Data);
}
