/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           0
#define CLOCK_EN              0xF
#define PLL_Engine_Enable     1 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x2

extern void P0_Loop(void); //GPIO1.c
extern void P1_Loop(void); //GPIO2.c
extern void P2_Loop(void); //GPIO3.c

/*-----------------------------------------------------------------------------
  Function:		TMR0_Delay1ms                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulCNT:	delay time, unit is millisecond.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 used for a delay function.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Delay1ms(uint32_t ulCNT)
{
    TCMPR0 = 12000;         //Fosc=12MHz, so 12000000/12000=1000Hz=1ms
    setTMR0_CEN;            //Start timer0
    while (ulCNT != 0)
    {
        while ((TISR0&TMR_TIF) != TMR_TIF); //check TIF0
        TISR0 |= TMR_TIF;   //Clear TIF0
        ulCNT --;
    }
    clrTMR0_CEN;            //Stop timer0
}

/*-----------------------------------------------------------------------------
  Function:		Timer0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0, period mode.                                    
 *-----------------------------------------------------------------------------*/
void Timer0_Init()
{
    TMR0_Clock_EN;
	TMR0ClkSource_ex12MHz;
    TCSR0  = 0x00000000;    //Pre-Scaler
    
    setTMR0_PERIOD;
    setTMR0_IE;             //Timer0 interrupt enable
    setTMR0_CRST;           //reset the timer/counter0, after set, this bit will be clear by H/W
}

main(void)
{
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

    Timer0_Init();

    P0_Loop();           //PC=0x4000, GPIO1.c
    P1_Loop();           //PC=0x8000, GPIO2.c
    P2_Loop();           //PC=0xC000, GPIO3.c


    while(1)
    {
        P2_DOUT = 0x00;            
        TMR0_Delay1ms(50);
        
        P2_DOUT = 0xFF;
        TMR0_Delay1ms(50);
    }
}
